package org.processmining.causalactivitygraphcreator.dialogs;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.model.XLog;
import org.processmining.causalactivitygraph.algorithms.VisualizeCausalActivityGraphAlgorithm;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraph.models.graph.CausalActivityGraphGraph;
import org.processmining.causalactivitygraphcreator.algorithms.DiscoverCausalActivityGraphAlgorithm;
import org.processmining.causalactivitygraphcreator.parameters.DiscoverCausalActivityGraphParameters;
import org.processmining.causalactivitymatrixminer.miners.MatrixMiner;
import org.processmining.causalactivitymatrixminer.miners.MatrixMinerManager;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.graphvisualizers.algorithms.GraphVisualizerAlgorithm;
import org.processmining.log.dialogs.ClassifierPanel;
import org.processmining.log.dialogs.MinerPanel;
import org.processmining.log.parameters.UpdateParameter;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

public class DiscoverCausalActivityGraphDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4809548576121463873L;

	public DiscoverCausalActivityGraphDialog(final UIPluginContext context, final XLog log,
			final DiscoverCausalActivityGraphParameters parameters) {
		double size[][] = { { TableLayoutConstants.FILL, TableLayoutConstants.FILL, TableLayoutConstants.FILL },
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL, 30, 30, 30/*, 30*/ } };
		setLayout(new TableLayout(size));

		this.setDoubleBuffered(true);

		List<String> miners = new ArrayList<String>();
		for (MatrixMiner miner : MatrixMinerManager.getInstance().getMiners()) {
			miners.add(miner.getName());
		}

		final JPanel panel = new JPanel();
		double panelSize[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL } };
		panel.setLayout(new TableLayout(panelSize));
		add(panel, "1, 0, 2, 4");
		updateGraph(context, panel, log, parameters);

		UpdateParameter updateParameter = new UpdateParameter() {
			public void update() {
				updateGraph(context, panel, log, parameters);
			}
		};

		if (parameters.isShowClassifierPanel()) {
			add(new MinerPanel(miners, parameters, updateParameter), "0, 1");
			add(new ClassifierPanel(log.getClassifiers(), parameters, updateParameter), "0, 0");
		} else {
			add(new MinerPanel(miners, parameters, updateParameter), "0, 0, 0, 1");
		}

		final JCheckBox checkBox = SlickerFactory.instance().createCheckBox("Use dot to preview graph", parameters.isUseDot());
		checkBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				parameters.setUseDot(checkBox.isSelected());
				updateGraph(context, panel, log, parameters);
			}

		});
		checkBox.setOpaque(false);
		add(checkBox, "0, 4");

		final NiceSlider zeroSlider = SlickerFactory.instance().createNiceDoubleSlider("Zero Value", -1.0, 1.0,
				parameters.getZeroValue(), Orientation.HORIZONTAL);
		zeroSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setZeroValue(-1.0 + zeroSlider.getSlider().getValue() * 2.0 / 10000.0);
				updateGraph(context, panel, log, parameters);
			}
		});
		zeroSlider.setPreferredSize(new Dimension(100,30));
		add(zeroSlider, "0, 2");

		final NiceSlider concSlider = SlickerFactory.instance().createNiceDoubleSlider("Concurrency Ratio", 0.0, 2.0,
				parameters.getConcurrencyRatio(), Orientation.HORIZONTAL);
		concSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setConcurrencyRatio(concSlider.getSlider().getValue() * 2.0 / 10000.0);
				updateGraph(context, panel, log, parameters);
			}
		});
		concSlider.setPreferredSize(new Dimension(100,30));
		add(concSlider, "0, 3");

//		final NiceSlider bestSlider = SlickerFactory.instance().createNiceDoubleSlider("Include Threshold", 0.0, 2.0,
//				parameters.getIncludeThreshold(), Orientation.HORIZONTAL);
//		bestSlider.addChangeListener(new ChangeListener() {
//
//			public void stateChanged(ChangeEvent e) {
//				parameters.setIncludeThreshold(bestSlider.getSlider().getValue() * 2.0 / 10000.0);
//				updateGraph(context, panel, log, parameters);
//			}
//		});
//		bestSlider.setPreferredSize(new Dimension(100,30));
//		add(bestSlider, "0, 5");

	}

	private void updateGraph(UIPluginContext context, JPanel panel, XLog log,
			DiscoverCausalActivityGraphParameters parameters) {
		DiscoverCausalActivityGraphAlgorithm algorithm = new DiscoverCausalActivityGraphAlgorithm();
		CausalActivityGraph graph = algorithm.apply(context, log, parameters);
		JComponent component = null;

		if (parameters.isUseDot()) {
			GraphVisualizerAlgorithm visualizer = new GraphVisualizerAlgorithm();
			component = visualizer.apply(context, new CausalActivityGraphGraph(context, graph));
		} else {
			VisualizeCausalActivityGraphAlgorithm visualizer = new VisualizeCausalActivityGraphAlgorithm();
			component = visualizer.apply(context, graph);
		}
		panel.removeAll();
		panel.add(component, "0, 0");
		panel.revalidate();
		panel.repaint();
	}
}
