package org.processmining.causalactivitygraphcreator.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraphcreator.algorithms.ConvertCausalActivityMatrixToCausalActivityGraphAlgorithm;
import org.processmining.causalactivitygraphcreator.connections.ConvertCausalActivityMatrixToCausalActivityGraphConnection;
import org.processmining.causalactivitygraphcreator.dialogs.ConvertCausalActivityMatrixToCausalActivityGraphDialog;
import org.processmining.causalactivitygraphcreator.help.ConvertCausalActivityMatrixToCausalActivityGraphHelp;
import org.processmining.causalactivitygraphcreator.parameters.ConvertCausalActivityMatrixToCausalActivityGraphParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Create Graph", parameterLabels = { "Causal Activity Matrix", "Parameters" }, returnLabels = { "Causal Activity Graph" }, returnTypes = { CausalActivityGraph.class }, help = ConvertCausalActivityMatrixToCausalActivityGraphHelp.TEXT)
public class ConvertCausalActivityMatrixToCausalActivityGraphPlugin extends
		ConvertCausalActivityMatrixToCausalActivityGraphAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Filter Graph, UI", requiredParameterLabels = { 0 })
	public CausalActivityGraph runUI(UIPluginContext context, CausalActivityMatrix matrix) {
		//		ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters = new ConvertCausalActivityMatrixToCausalActivityGraphParameters();
		ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters = optParameters(context, matrix, 100, 100);
		ConvertCausalActivityMatrixToCausalActivityGraphDialog dialog = new ConvertCausalActivityMatrixToCausalActivityGraphDialog(
				context, matrix, parameters);
		InteractionResult result = context.showWizard("Configure filter (thresholds)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, matrix, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public CausalActivityGraph convertUI(UIPluginContext context, CausalActivityMatrix matrix) {
		return runUI(context, matrix);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Graph, Default", requiredParameterLabels = { 0 })
	public CausalActivityGraph runDefault(PluginContext context, CausalActivityMatrix matrix) {
		ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters = new ConvertCausalActivityMatrixToCausalActivityGraphParameters();
		return runConnection(context, matrix, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public CausalActivityGraph convertDefault(PluginContext context, CausalActivityMatrix matrix) {
		return runDefault(context, matrix);
	}

	@PluginVariant(variantLabel = "Create Graph, Parameters", requiredParameterLabels = { 0 })
	public CausalActivityGraph run(PluginContext context, CausalActivityMatrix matrix,
			ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters) {
		return runConnection(context, matrix, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public CausalActivityGraph convertParameters(PluginContext context, CausalActivityMatrix matrix,
			ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters) {
		return run(context, matrix, parameters);
	}

	private CausalActivityGraph runConnection(PluginContext context, CausalActivityMatrix matrix,
			ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ConvertCausalActivityMatrixToCausalActivityGraphConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						ConvertCausalActivityMatrixToCausalActivityGraphConnection.class, context, matrix);
				for (ConvertCausalActivityMatrixToCausalActivityGraphConnection connection : connections) {
					if (connection.getObjectWithRole(ConvertCausalActivityMatrixToCausalActivityGraphConnection.MATRIX)
							.equals(matrix) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(ConvertCausalActivityMatrixToCausalActivityGraphConnection.GRAPH);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		CausalActivityGraph graph = apply(context, matrix, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new ConvertCausalActivityMatrixToCausalActivityGraphConnection(matrix, graph, parameters));
		}
		return graph;
	}

	@Deprecated
	public ConvertCausalActivityMatrixToCausalActivityGraphParameters optParameters(PluginContext context,
			CausalActivityMatrix matrix, int nofZeroSteps, int nofConcSteps) {
		ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters = new ConvertCausalActivityMatrixToCausalActivityGraphParameters();
		return parameters;
	}
}
