package org.processmining.causalactivitygraphcreator.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraphcreator.algorithms.DiscoverCausalActivityGraphAlgorithm;
import org.processmining.causalactivitygraphcreator.dialogs.DiscoverCausalActivityGraphDialog;
import org.processmining.causalactivitygraphcreator.help.DiscoverCausalActivityGraphHelp;
import org.processmining.causalactivitygraphcreator.parameters.DiscoverCausalActivityGraphParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Discover Graph", parameterLabels = { "Log", "Parameters" }, returnLabels = { "Causal Activity Graph" }, returnTypes = { CausalActivityGraph.class }, help = DiscoverCausalActivityGraphHelp.TEXT)
public class DiscoverCausalActivityGraphPlugin extends DiscoverCausalActivityGraphAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Discover Graph, UI", requiredParameterLabels = { 0 })
	public CausalActivityGraph runUI(UIPluginContext context, XLog log) {
		DiscoverCausalActivityGraphParameters parameters = new DiscoverCausalActivityGraphParameters(log);
		DiscoverCausalActivityGraphDialog dialog = new DiscoverCausalActivityGraphDialog(
				context, log, parameters);
		InteractionResult result = context.showWizard("Configure discoverer", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		return runConnection(context, log, parameters);
	}

	@PluginVariant(variantLabel = "Discover Graph, Parameters", requiredParameterLabels = { 0 })
	public CausalActivityGraph run(PluginContext context, XLog log,
			DiscoverCausalActivityGraphParameters parameters) {
		return runConnection(context, log, parameters);
	}

	private CausalActivityGraph runConnection(PluginContext context, XLog log,
			DiscoverCausalActivityGraphParameters parameters) {
		CausalActivityGraph graph = apply(context, log, parameters);
		return graph;
	}
}
